{{ ************************************
   *  Calendar Demo                   *
   *  (C) 2007 Gary Noel Boone, PhD   *
   ************************************

  This module demonstrates how to call the two public functions of the Calendar module:

        ConvertUTCtoLocaltime()
        DateOfNthMday()

  These functions are demonstrated using a simple unit test system. If all of the tests
  pass, the program prints simply "Tests completed." Any tests that fail, however, print
  which test failed and the expected and actual values. 

  The program prints results to the TV output using the terminal object.
}}


CON

  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000


OBJ
  Term  : "tv_terminal"
  Cal   : "Calendar"

            
PUB Start
  ' initialize
  Term.start(12)                                        ' Start TV Terminal Object
  SetScreenWhiteOnDarkBlue

  ' Unit tests
  RunNthDayUnitTests
  RunUTCConversionUnitTests

  ' Done  
  Term.str(STRING("Tests completed."))
  Term.out(13)                                        ' carriage return




PRI SetScreenWhiteOnDarkBlue                 
'' Set the screen background and foreground colors
  Term.Out(3)                       
  Term.Out(5)


PRI RunUTCConversionUnitTests | dateStr, timeStr
'' These unit tests aren't included in the calendar spin file to avoid increasing its size when deployed.
                                
  ' 0 offset
  Cal.ConvertUTCtoLocaltime(0, 2007, 6, 2, 1, 34, 10, FALSE, @dateStr, @timeStr)     ' no offset, no DST adj
  AssertStringEqual(STRING("06/02/07"), dateStr, STRING("UTCConv Test 0.0")) 
  AssertStringEqual(STRING("01:34:10"), timeStr, STRING("UTCConv Test 0.1")) 
  Cal.ConvertUTCtoLocaltime(0, 2007, 6, 2, 1, 34, 10, TRUE, @dateStr, @timeStr)      ' no offset, but DST
  AssertStringEqual(STRING("06/02/07"), dateStr, STRING("UTCConv Test 1.0")) 
  AssertStringEqual(STRING("00:34:10"), timeStr, STRING("UTCConv Test 1.1"))                          
  ' small offset
  Cal.ConvertUTCtoLocaltime(-2, 2007, 6, 2, 1, 34, 10, FALSE, @dateStr, @timeStr)
  AssertStringEqual(STRING("06/01/07"), dateStr, STRING("UTCConv Test 2.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 2.1")) 
  Cal.ConvertUTCtoLocaltime(-2, 2007, 6, 2, 1, 34, 10, TRUE, @dateStr, @timeStr)     ' ahead w/DST
  AssertStringEqual(STRING("06/02/07"), dateStr, STRING("UTCConv Test 3.0")) 
  AssertStringEqual(STRING("00:34:10"), timeStr, STRING("UTCConv Test 3.1")) 

  ''A. sequence from 7 to 11 UTC on a non-DST, non-changing day
  Cal.ConvertUTCtoLocaltime(-8, 2007, 2, 1, 7, 34, 10, FALSE, @dateStr, @timeStr)    ' no DST adj., so simple -8
  AssertStringEqual(STRING("01/31/07"), dateStr, STRING("UTCConv Test 4.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 4.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 2, 1, 7, 34, 10, TRUE, @dateStr, @timeStr)     ' DST adj., but Feb, so -8
  AssertStringEqual(STRING("01/31/07"), dateStr, STRING("UTCConv Test 5.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 5.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 2, 1, 8, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("02/01/07"), dateStr, STRING("UTCConv Test 6.0")) 
  AssertStringEqual(STRING("00:34:10"), timeStr, STRING("UTCConv Test 6.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 2, 1, 9, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("02/01/07"), dateStr, STRING("UTCConv Test 7.0")) 
  AssertStringEqual(STRING("01:34:10"), timeStr, STRING("UTCConv Test 7.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 2, 1, 10, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("02/01/07"), dateStr, STRING("UTCConv Test 8.0")) 
  AssertStringEqual(STRING("02:34:10"), timeStr, STRING("UTCConv Test 8.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 2, 1, 11, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("02/01/07"), dateStr, STRING("UTCConv Test 9.0")) 
  AssertStringEqual(STRING("03:34:10"), timeStr, STRING("UTCConv Test 9.1")) 

  ''B. sequence from 6 to 10 UTC on a DST, non-changing day
  Cal.ConvertUTCtoLocaltime(-8, 2007, 8, 1, 6, 34, 10, FALSE, @dateStr, @timeStr)    ' no DST adj., so simple -8
  AssertStringEqual(STRING("07/31/07"), dateStr, STRING("UTCConv Test 10.0")) 
  AssertStringEqual(STRING("22:34:10"), timeStr, STRING("UTCConv Test 10.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 8, 1, 6, 34, 10, TRUE, @dateStr, @timeStr)     ' DST adj., so +1 hour
  AssertStringEqual(STRING("07/31/07"), dateStr, STRING("UTCConv Test 11.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 11.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 8, 1, 7, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("08/01/07"), dateStr, STRING("UTCConv Test 12.0")) 
  AssertStringEqual(STRING("00:34:10"), timeStr, STRING("UTCConv Test 12.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 8, 1, 8, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("08/01/07"), dateStr, STRING("UTCConv Test 13.0")) 
  AssertStringEqual(STRING("01:34:10"), timeStr, STRING("UTCConv Test 13.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 8, 1, 9, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("08/01/07"), dateStr, STRING("UTCConv Test 14.0")) 
  AssertStringEqual(STRING("02:34:10"), timeStr, STRING("UTCConv Test 14.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 8, 1, 10, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("08/01/07"), dateStr, STRING("UTCConv Test 15.0")) 
  AssertStringEqual(STRING("03:34:10"), timeStr, STRING("UTCConv Test 15.1")) 


  ''C. sequence from 7 to 11 UTC on a DST-begin day, the 2nd Sunday in March
  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 11, 7, 34, 10, FALSE, @dateStr, @timeStr)   ' no DST adj., so simple -8
  AssertStringEqual(STRING("03/10/07"), dateStr, STRING("UTCConv Test 16.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 16.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 11, 7, 34, 10, TRUE, @dateStr, @timeStr)    ' DST adj., but pre-DST, so -8
  AssertStringEqual(STRING("03/10/07"), dateStr, STRING("UTCConv Test 17.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 17.1"))  
  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 11, 8, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("03/11/07"), dateStr, STRING("UTCConv Test 18.0")) 
  AssertStringEqual(STRING("00:34:10"), timeStr, STRING("UTCConv Test 18.1"))   
  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 11, 9, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("03/11/07"), dateStr, STRING("UTCConv Test 19.0")) 
  AssertStringEqual(STRING("01:34:10"), timeStr, STRING("UTCConv Test 19.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 11, 10, 34, 10, TRUE, @dateStr, @timeStr)   ' change to DST, spring forward                      
  AssertStringEqual(STRING("03/11/07"), dateStr, STRING("UTCConv Test 20.0"))  
  AssertStringEqual(STRING("03:34:10"), timeStr, STRING("UTCConv Test 20.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 11, 11, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("03/11/07"), dateStr, STRING("UTCConv Test 21.0")) 
  AssertStringEqual(STRING("04:34:10"), timeStr, STRING("UTCConv Test 21.1"))

  Cal.ConvertUTCtoLocaltime(-8, 2007, 3, 12, 6, 34, 10, TRUE, @dateStr, @timeStr)    ' next day, now DST, so -7                        
  AssertStringEqual(STRING("03/11/07"), dateStr, STRING("UTCConv Test 22.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 22.1"))

  Cal.ConvertUTCtoLocaltime(-8, 2007, 4, 2, 6, 34, 10, TRUE, @dateStr, @timeStr)     ' next month, now DST, so -7                        
  AssertStringEqual(STRING("04/01/07"), dateStr, STRING("UTCConv Test 23.0"))  
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 23.1")) 

  ''D. sequence from 6 to 10 UTC on a DST-end day, the 1st Sunday in November
  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 4, 6, 34, 10, FALSE, @dateStr, @timeStr)   ' no DST adj., so simple -8
  AssertStringEqual(STRING("11/03/07"), dateStr, STRING("UTCConv Test 24.0")) 
  AssertStringEqual(STRING("22:34:10"), timeStr, STRING("UTCConv Test 24.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 4, 6, 34, 10, TRUE, @dateStr, @timeStr)    ' DST adj., so +1 hour
  AssertStringEqual(STRING("11/03/07"), dateStr, STRING("UTCConv Test 25.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 25.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 4, 7, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("11/04/07"), dateStr, STRING("UTCConv Test 26.0")) 
  AssertStringEqual(STRING("00:34:10"), timeStr, STRING("UTCConv Test 26.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 4, 8, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("11/04/07"), dateStr, STRING("UTCConv Test 27.0")) 
  AssertStringEqual(STRING("01:34:10"), timeStr, STRING("UTCConv Test 27.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 4, 9, 34, 10, TRUE, @dateStr, @timeStr)    ' change to non-DST, fall back
  AssertStringEqual(STRING("11/04/07"), dateStr, STRING("UTCConv Test 28.0")) 
  AssertStringEqual(STRING("01:34:10"), timeStr, STRING("UTCConv Test 28.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 4, 10, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("11/04/07"), dateStr, STRING("UTCConv Test 29.0")) 
  AssertStringEqual(STRING("02:34:10"), timeStr, STRING("UTCConv Test 29.1")) 

  Cal.ConvertUTCtoLocaltime(-8, 2007, 11, 5, 7, 34, 10, TRUE, @dateStr, @timeStr)    ' next day, now ST, so -8
  AssertStringEqual(STRING("11/04/07"), dateStr, STRING("UTCConv Test 30.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 30.1")) 
  Cal.ConvertUTCtoLocaltime(-8, 2007, 12, 5, 7, 34, 10, TRUE, @dateStr, @timeStr)    ' next month, now ST, so -8
  AssertStringEqual(STRING("12/04/07"), dateStr, STRING("UTCConv Test 31.0")) 
  AssertStringEqual(STRING("23:34:10"), timeStr, STRING("UTCConv Test 31.1")) 


  ' check leap day
  Cal.ConvertUTCtoLocaltime(-8, 2008, 3, 1, 6, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("02/29/08"), dateStr, STRING("UTCConv Test 32.0")) 
  AssertStringEqual(STRING("22:34:10"), timeStr, STRING("UTCConv Test 32.1")) 
  ' check year rollback
  Cal.ConvertUTCtoLocaltime(-8, 2008, 1, 1, 6, 34, 10, TRUE, @dateStr, @timeStr)
  AssertStringEqual(STRING("12/31/07"), dateStr, STRING("UTCConv Test 33.0")) 
  AssertStringEqual(STRING("22:34:10"), timeStr, STRING("UTCConv Test 33.1")) 


  
PRI RunNthDayUnitTests
'' These unit tests aren't included in the calendar spin file to avoid increasing its size when deployed.
  AssertEqual(23, Cal.DateOfNthMday(4, Cal#Tue, Cal#Dec, 1845), STRING("NthDay Test0"))
  AssertEqual( 2, Cal.DateOfNthMday(1, Cal#Sat, Cal#Dec, 1967), STRING("NthDay Test1"))
  AssertEqual( 4, Cal.DateOfNthMday(1, Cal#Fri, Cal#Aug, 2000), STRING("NthDay Test2"))  ' 2000 was a leap year
  AssertEqual( 4, Cal.DateOfNthMday(1, Cal#Wed, Cal#Aug, 2004), STRING("NthDay Test3"))  ' 2004 was a leap year
  AssertEqual(17, Cal.DateOfNthMday(3, Cal#Thu, Cal#Dec, 1903), STRING("NthDay Test4"))
  
  ' Check the Wednesdays of a leap year January
  AssertEqual( 7, Cal.DateOfNthMday(1, Cal#Wed, Cal#Jan, 2004), STRING("NthDay Test5"))  ' 2004 was a leap year
  AssertEqual(14, Cal.DateOfNthMday(2, Cal#Wed, Cal#Jan, 2004), STRING("NthDay Test6"))
  AssertEqual(21, Cal.DateOfNthMday(3, Cal#Wed, Cal#Jan, 2004), STRING("NthDay Test7"))
  AssertEqual(28, Cal.DateOfNthMday(4, Cal#Wed, Cal#Jan, 2004), STRING("NthDay Test8"))
  
  ' check all the Saturdays of a non-leap year
  AssertEqual( 1, Cal.DateOfNthMday(1, Cal#Sat, Cal#Jan, 1955), STRING("NthDay Test9"))
  AssertEqual( 8, Cal.DateOfNthMday(2, Cal#Sat, Cal#Jan, 1955), STRING("NthDay Test10"))
  AssertEqual(15, Cal.DateOfNthMday(3, Cal#Sat, Cal#Jan, 1955), STRING("NthDay Test11"))
  AssertEqual(22, Cal.DateOfNthMday(4, Cal#Sat, Cal#Jan, 1955), STRING("NthDay Test12"))
  AssertEqual(29, Cal.DateOfNthMday(5, Cal#Sat, Cal#Jan, 1955), STRING("NthDay Test13"))
  ' note that the library will not return a day higher than allowed in the month
  AssertEqual(-1, Cal.DateOfNthMday(6, Cal#Sat, Cal#Jan, 1955), STRING("NthDay Test14"))
     
  ' this month
  AssertEqual(22, Cal.DateOfNthMday(4, Cal#Fri, Cal#Jun, 2007), STRING("NthDay Test15"))
  ' this New Year's
  AssertEqual( 1, Cal.DateOfNthMday(1, Cal#Mon, Cal#Jan, 2007), STRING("NthDay Test16"))
  ' this year's leap day?
  AssertEqual(-1, Cal.DateOfNthMday(5, Cal#Thu, Cal#Feb, 2007), STRING("NthDay Test17")) ' no
    
  ' Feb 2004, a leap year  
  AssertEqual(22, Cal.DateOfNthMday(4, Cal#Sun, Cal#Feb, 2004), STRING("NthDay Test18"))
  AssertEqual(29, Cal.DateOfNthMday(5, Cal#Sun, Cal#Feb, 2004), STRING("NthDay Test19")) ' leap day
  
    
  AssertEqual(29, Cal.DateOfNthMday(5, Cal#Fri, Cal#Feb, 2008), STRING("NthDay Test20")) ' next leap day
  
  ' This year's DST change dates
  AssertEqual(11, Cal.DateOfNthMday(2, Cal#Sun, Cal#Mar, 2007), STRING("NthDay Test21")) ' spring forward
  AssertEqual(4, Cal.DateOfNthMday(1, Cal#Sun, Cal#Nov, 2007), STRING("NthDay Test22"))  ' fall back
  ' Next year's DST change dates
  AssertEqual(9, Cal.DateOfNthMday(2, Cal#Sun, Cal#Mar, 2008), STRING("NthDay Test23"))  ' spring forward
  AssertEqual(2, Cal.DateOfNthMday(1, Cal#Sun, Cal#Nov, 2008), STRING("NthDay Test24"))  ' fall back
  ' 2009 DST change dates
  AssertEqual(8, Cal.DateOfNthMday(2, Cal#Sun, Cal#Mar, 2009), STRING("NthDay Test25"))  ' spring forward
  AssertEqual(1, Cal.DateOfNthMday(1, Cal#Sun, Cal#Nov, 2009), STRING("NthDay Test26"))  ' fall back
  ' 2010 DST change dates
  AssertEqual(14, Cal.DateOfNthMday(2, Cal#Sun, Cal#Mar, 2010), STRING("NthDay Test27")) ' spring forward
  AssertEqual(7, Cal.DateOfNthMday(1, Cal#Sun, Cal#Nov, 2010), STRING("NthDay Test28"))  ' fall back


  
PUB AssertEqual(x, y, msg)
'' Assert function for values. Pass in the expected value and the result of the function tested.
'' Make msg uniquely identify the assert call.
  if x <> y
    Term.str(STRING("Failure: Exp. "))
    Term.dec(x)
    Term.str(STRING(", Rec'd "))
    Term.dec(y)
    Term.str(STRING(". "))                 
    Term.str(msg)
    Term.out(13)                                        ' carriage return
    return FALSE
  return TRUE

PUB AssertStringEqual(str1, str2, msg)
'' Assert function for strings. Pass in the expected value and the result of the function tested.
'' Make msg uniquely identify the assert call.
  if !STRCOMP(str1, str2)
    Term.str(STRING("Failure: "))
    Term.str(msg)
    Term.out(13)                                        ' carriage return
    Term.str(STRING("Exp.: '"))
    Term.str(str1)
    Term.str(STRING("', Rec'd: '"))
    Term.str(str2)
    Term.str(STRING("'. "))                 
    Term.out(13)                                        ' carriage return
    return FALSE
  return TRUE
           